package com.xm.test.common.auth.aspect;

import com.xm.test.common.auth.AuthFun;
import com.xm.test.common.auth.ClassUtil;
import com.xm.test.common.auth.annotation.PreAuth;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.expression.BeanFactoryResolver;
import org.springframework.core.MethodParameter;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;

/**
 * AOP权限校验
 *
 * @author mengxc
 * @date 2024/1/2 13:14
 */
@Aspect
@Component
@Slf4j
public class PreAuthAspect {

    /**
     * SPEL验证器
     */
    private static final ExpressionParser EXPRESSION_PARSER = new SpelExpressionParser();

    @Autowired
    private ApplicationContext applicationContext;

    @Around("@annotation(com.xm.test.common.auth.annotation.PreAuth)||@within(com.xm.test.common.auth.annotation.PreAuth)")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        if (handleAuth(point)) {
            return point.proceed();
        }
        // 应该抛出暂无权限的自定义异常 由全局异常处理器捕捉处理
        throw new Exception();
    }

    /**
     * 权限验证处理
     *
     * @param point 切点
     * @return 权限验证结果
     */
    private boolean handleAuth(ProceedingJoinPoint point) {
        MethodSignature ms = point.getSignature() instanceof MethodSignature ? (MethodSignature) point.getSignature() : null;
        assert ms != null;
        Method method = ms.getMethod();
        // 读取权限注解，优先方法上，没有则读取类
        PreAuth preAuth = ClassUtil.getAnnotation(method, PreAuth.class);
        // 判断表达式 为表达式，如：hasTimeAuth(1,10)
        String condition = preAuth.value();
        if (StringUtils.isNotBlank(condition)) {
            Expression expression = EXPRESSION_PARSER.parseExpression(condition);
            // 方法参数值
            Object[] args = point.getArgs();
            // 方法映射
            StandardEvaluationContext context = this.getEvaluationContext(method, args);
            // 获取解析计算的结果
            return Boolean.TRUE.equals(expression.getValue(context, Boolean.class));
        }
        return false;
    }

    /**
     * 获取方法上的参数
     *
     * @param method 方法
     * @param args   变量
     * @return {SimpleEvaluationContext}
     */
    private StandardEvaluationContext getEvaluationContext(Method method, Object[] args) {
        // 初始化SPEL表达式上下文，并设置 AuthFun
        StandardEvaluationContext context = new StandardEvaluationContext(new AuthFun());
        // 设置表达式支持spring bean
        context.setBeanResolver(new BeanFactoryResolver(applicationContext));
        for (int i = 0; i < args.length; i++) {
            // 读取方法参数
            MethodParameter methodParam = ClassUtil.getMethodParameter(method, i);
            // 设置方法 参数名和值 为SPEL变量
            context.setVariable(methodParam.getParameterName(), args[i]);
        }
        return context;
    }
}
